<?php
/**
 * phpaga
 *
 * Bills
 *
 * This is the invoices (bills) management interface.
 *
 * @author Florian Lanthaler <florian@phpaga.net>
 * @version $Id: bills.php 753 2006-10-25 21:52:28Z florian $
 *
 * Copyright (c) 2002, Florian Lanthaler <florian@phpaga.net>
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *    * Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *    * Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in
 *      the documentation and/or other materials provided with the
 *      distribution.
 *
 *    * Neither the name of Florian Lanthaler nor the names of his
 *      contributors may be used to endorse or promote products derived
 *      from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

include_once("./config.php");

$perms = array(PHPAGA_PERM_VIEW_INVOICES, PHPAGA_PERM_MANAGE_INVOICES);
phpaga_user_pageperm($perms);

$search_args = "";
$rows = "";
$count = "";
$browse = "";
$select_company = "";
$select_paid = array(PHPAGA_BILL_PAID => _("paid"),
                     PHPAGA_BILL_NOTPAID => _("not paid"));
$offset = 0;
$billInfo = array('cpn_id' => '',
                  'bmt_id' => '',
                  'curr_id' => '',
                  'bill_number' => '',
                  'bill_paid' => '',
                  'bill_sent' => '',
                  'bill_datefrom' => '',
                  'bill_dateto' => '',
                  'bill_year' => null);

phpaga_arrayAddOption($select_paid, PHPAGA_OPTION_ALL);

if (isset($_REQUEST["offset"]) && is_numeric($_REQUEST["offset"]))
    $offset = $_REQUEST["offset"];

if (isset($_REQUEST["cpn_id"]) && is_numeric($_REQUEST["cpn_id"]))
    $billInfo["cpn_id"] = $_REQUEST["cpn_id"];

if (isset($_REQUEST["bmt_id"]) && is_numeric($_REQUEST["bmt_id"]))
    $billInfo["bmt_id"] = $_REQUEST["bmt_id"];

if (isset($_REQUEST["curr_id"]) && is_numeric($_REQUEST["curr_id"]))
    $billInfo["curr_id"] = $_REQUEST["curr_id"];

if (isset($_REQUEST["bill_number"]) && strlen ($_REQUEST["bill_number"]))
    $billInfo["bill_number"] = $_REQUEST["bill_number"];

if (isset($_REQUEST["bill_paid"]) && is_numeric($_REQUEST["bill_paid"]) &&
    (($_REQUEST["bill_paid"] ==  PHPAGA_BILL_PAID) ||
     ($_REQUEST["bill_paid"] == PHPAGA_BILL_NOTPAID)))

    $billInfo["bill_paid"] = $_REQUEST["bill_paid"];

if (isset($_REQUEST["bill_sent"]) && is_numeric($_REQUEST["bill_sent"]) &&
    (($_REQUEST["bill_sent"] == PHPAGA_BILL_SENT) ||
     ($_REQUEST["bill_sent"] == PHPAGA_BILL_NOTSENT)))

    $billInfo["bill_sent"] = $_REQUEST["bill_sent"];

if (isset($_REQUEST["bill_datefrom"]) && strlen($_REQUEST["bill_datefrom"]))
    $billInfo["bill_datefrom"] = rawurldecode($_REQUEST["bill_datefrom"]);

if (isset($_REQUEST["bill_dateto"]) && strlen($_REQUEST["bill_dateto"]))
    $billInfo["bill_dateto"] = rawurldecode($_REQUEST["bill_dateto"]);

foreach ($billInfo as $key =>$value) {
    if (isset($value) && strlen($value))
        $search_args .= "&amp;$key=".rawurlencode ($value);
}

$result = phpaga_bills_search ($rows, $count, $billInfo, $offset, PHPAGA_RECORDS_PERPAGE);

phpaga_header(array("calendar" => true, "menuitem" => "finance"));

if ($result != PHPAGA_ERR_NONE) {
    phpaga_error ($result);
}

phpaga_companies_getCompanyList($select_company);
phpaga_arrayAddOption($select_company, PHPAGA_OPTION_ALL);

$bill_tpl = new psmarty;

$bill_tpl->assign("perm_invoices", phpaga_user_hasperm(PHPAGA_PERM_MANAGE_INVOICES));
$bill_tpl->assign("FORM_ACTION", basename($_SERVER["PHP_SELF"]));
$bill_tpl->assign("billInfo", $billInfo);
$bill_tpl->assign("select_cpn", $select_company);
$bill_tpl->assign("select_paid", $select_paid);
$bill_tpl->assign("search_args", $search_args);
$bill_tpl->assign("rows", $rows);
$bill_tpl->assign("BROWSE",
                  phpaga_navigate(basename($_SERVER["PHP_SELF"]). "?$search_args",
                                  $count,
                                  $offset, PHPAGA_RECORDS_PERPAGE));

$bill_tpl->display("bills.tpl.html");

phpaga_footer();

?>