<?php
/**
 * phpaga
 *
 * Currencies
 *
 * This is the currencies management interface.
 *
 * @author Florian Lanthaler <florian@phpaga.net>
 * @version $Id: currencies.php 726 2006-08-05 22:34:30Z florian $
 *
 * Copyright (c) 2002, Florian Lanthaler <florian@phpaga.net>
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *    * Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *    * Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in
 *      the documentation and/or other materials provided with the
 *      distribution.
 *
 *    * Neither the name of Florian Lanthaler nor the names of his
 *      contributors may be used to endorse or promote products derived
 *      from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

include_once ("./config.php");
phpaga_user_pageperm(PHPAGA_PERM_MANAGE_SYSSETTINGS);

$rows = array();
$curr_name = "";
$curr_decimals = "";

phpaga_header(array("menuitem" => "administration"));

if (isset ($_REQUEST["action"]) && strlen ($_REQUEST["action"]))
{

    switch ($_REQUEST["action"])
    {

        case ACTION_ADD:

            /* add a new currency */

            if (isset($_REQUEST["curr_name"])
                && strlen($_REQUEST["curr_name"])
                && isset($_REQUEST["curr_decimals"])
                && is_numeric($_REQUEST["curr_decimals"]))
            {
                $result = phpaga_currencies_addCurrency($_REQUEST["curr_name"],
                                                        $_REQUEST["curr_decimals"]);

                if ($result != PHPAGA_ERR_NONE)
                {
                    phpaga_error ($result);
                    $curr_name = $_REQUEST["curr_name"];
                    $curr_decimals = $_REQUEST["curr_decimals"];
                }
                else
                {
                    phpaga_message (_("Currency added"),
                                    _("The currency has been inserted successfully."));
                }
            }
            else
            {
                phpaga_message (_("Unable to add currency"),
                                _("The currency can not be added - please check if you gave it a valid name and number of decimals."));
                $curr_name = $_REQUEST["curr_name"];
                $curr_decimals = $_REQUEST["curr_decimals"];
            }
            break;

        case ACTION_DELETE:

            /* delete a currency */

            if (isset ($_REQUEST["id"]) && strlen ($_REQUEST["id"]))
            {
                $result = phpaga_currencies_deleteCurrency ($_REQUEST["id"]);

                if ($result != PHPAGA_ERR_NONE)
                {

                    /* if we could not delete the record because of
                     * dependencies then use a more descriptive
                     * error message
                     */

                    if ($result == PHPAGA_ERR_DEL_DEPENDENCIES)
                    {
                        $result =
                            _("The currency cannot be deleted, as it is still used by some bills.");
                    }

                    phpaga_error ($result);
                }
                else
                {
                    phpaga_message(_("Currency deleted"),
                                   _("The currency has been successfully deleted."));
                }
            }
            break;


        default:
            break;
    }
}

$result = phpaga_currencies_getCurrencies ($rows);

if ($result != PHPAGA_ERR_NONE)
{
    phpaga_error ($result);
}

$currencies_tpl = new psmarty;

$currencies_tpl->assign ("FORM_ACTION",
                         basename ($_SERVER["PHP_SELF"]));

$currencies_tpl->assign ("curr_name", $curr_name);
$currencies_tpl->assign ("curr_decimals", $curr_decimals);
$currencies_tpl->assign ("curr_decimals_options", $phpaga_decimals_options);
$currencies_tpl->assign ("rows", $rows);

$currencies_tpl->display ("currencies.tpl.html");

phpaga_footer();

?>