<?php
  /**
   * phpaga
   *
   * file preview
   *
   * Preview a file that has been associated to an internal object
   * (for example a project, a task, a company, a person, ...)
   *
   * @author Florian Lanthaler <florian@phpaga.net>
   * @version $Id: preview.php 746 2006-09-27 21:21:34Z florian $
   *
   * Copyright (c) 2006, Florian Lanthaler <florian@phpaga.net>
   *
   * All rights reserved.
   *
   * Redistribution and use in source and binary forms, with or without
   * modification, are permitted provided that the following conditions are
   * met:
   *
   *    * Redistributions of source code must retain the above copyright
   *      notice, this list of conditions and the following disclaimer.
   *
   *    * Redistributions in binary form must reproduce the above copyright
   *      notice, this list of conditions and the following disclaimer in
   *      the documentation and/or other materials provided with the
   *      distribution.
   *
   *    * Neither the name of Florian Lanthaler nor the names of his
   *      contributors may be used to endorse or promote products derived
   *      from this software without specific prior written permission.
   *
   * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
   * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
   * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
   * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
   * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
   * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
   * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
   * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
   * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
   * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
   * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
   */

include_once("./config.php");
require_once('HTTP/Download.php');

ob_start();

if (!isset($_REQUEST["id"]) || !is_numeric($_REQUEST["id"])) {
    phpaga_error(_("Invalid file ID."));
    exit;
}

$file = array();

$result = phpaga_files_getInfo($_REQUEST["id"], $file);

if (!in_array($file["file_mimetype"], $phpaga_preview_types))
    exit;

if ($result != PHPAGA_ERR_NONE) {
    phpaga_error($result);
    exit;
}

switch ($file["file_rel_type"])
{
case PHPAGA_RELTYPE_PROJECT:

    if (!phpaga_project_canview($file["file_rel_id"]))
    {
        phpaga_error(_("You are not allowed to access this file."));
        exit;
    }

default:
    break;

}

$dest_dir = PHPAGA_FILEPATH.$file["file_rel_type"]."/".$file["file_rel_id"]."/";

$thumb_dir = $dest_dir.".thumbs/";

if (!is_dir($thumb_dir) && !mkdir($thumb_dir))
{
    phpaga_error(_("Unable to create thumbnail directory"));
    exit;
}


if (file_exists($thumb_dir.$file["file_local_name"]) &&
    (filemtime($thumb_dir.$file["file_local_name"]) < filemtime($dest_dir.$file["file_local_name"])))

    unlink($thumb_dir.$file["file_local_name"]);

$thumb_file = $thumb_dir.$file["file_local_name"];
if (in_array($file["file_mimetype"], $phpaga_preview_types_tojpg))
    $thumb_file .= ".jpg";

if (!file_exists($thumb_dir.$file["file_local_name"])) {

    $orig_file = $dest_dir.$file["file_local_name"];

    if (in_array($file["file_mimetype"], $phpaga_preview_types_firstpage))
        $orig_file .="[0]"; // first page only

    $command = sprintf("%s -size 100x100 %s -resize 100x100 %s",
                       PHPAGA_BIN_CONVERT,
                       escapeshellcmd($orig_file),
                       escapeshellcmd($thumb_file));
    $ex = `$command`;
}

$dl = new HTTP_Download();
$dl->setFile($thumb_file);
$dl->setContentDisposition(HTTP_DOWNLOAD_INLINE);
//$dl->setContentType($file["file_mimetype"]);
$dl->send();

?>