<?php
/**
 * phpaga
 *
 * billing plugin
 *
 * This plugin contains the necessary routines to calculate a bill for:
 *
 * Spanish IVA/ calculo en Euros
 *
 * Original version :
 * @author Daniel Cabezas <daniel@seriousworks.net>
 * @version $Id: esp_liberoprof_euro.php 653 2005-10-03 21:53:26Z worf $
 *
 * Copyright (c) 2003, Daniel Cabezas <daniel@seriousworks.net>
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *    * Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *    * Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in
 *      the documentation and/or other materials provided with the
 *      distribution.
 *
 *    * Neither the name of Florian Lanthaler nor the names of his
 *      contributors may be used to endorse or promote products derived
 *      from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
/**
 * Calculates the amount to be paid based on a startsum according to the
 * specified rules. Returns also an array containing all the documented
 * steps for this kind of billing method.
 * This method is used to create invoices in Euro as it respects the
 * particular rounding function needed.
 *
 * @param real   $startsum  Start amount as base of calculation
 * @param int    $curr_id   Currency-ID
 * @param real   $endsum    The calculated amount to be paid
 * @param array  $details   Billing details
 *
 * @return int   $result    A result code. 
 *
 * @author Florian Lanthaler <florian@phpaga.net>
 * @since phpaga 0.1
 * @author Daniel Cabezas <daniel@seriousworks.net>
 * for release 24-08-2003
 *
 */
 
function phpaga_bill_calculate ($startsum, 
                                $curr_id = "", 
                                &$endsum, 
                                &$details)
{
    $details = new phpaga_bill_details ($curr_id);

    $endsum = 0;
    $inps = 0;
    $imponibile = 0;
    $iva = 0;
    $irpef = 0;

    /* 16% IVA */
    $inps = sprintf ("%.2f", $startsum * 0.16);

    /* imponibile */
    $imponibile = sprintf ("%.2f", $startsum + $inps);

    /* IVA 20 % */
    $iva = sprintf ("%.2f", $imponibile * 0.2);

    /* IRPEF 20 % */
    $irpef = sprintf ("%.2f", $imponibile * 0.2);

    /* end sum */
    $endsum = sprintf ("%.2f", $imponibile + $iva - $irpef);

    $details->add_detail ("Subtotal", $startsum);
    $details->add_detail ("16% IVA", $inps);
    $details->add_separator ();
    $details->add_detail ("Total", $endsum);

    return PHPAGA_ERR_NONE;
}

?>
